using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

static class Program
{ [STAThread] static void Main() { Game1 game = new Game1(); game.Run(); }

public class Game1 : Game
{ private GraphicsDeviceManager g;
  private ContentManager content;
  private Model skyModel, shipModel;
  private BasicEffect skyEffect;
  //private Matrix world, view, proj;
  private Matrix viewM, projM, skyworldM, shipworldM;
  private static float skyscale  = 10000f;
  private float slow = skyscale / 200f;  // step width of movements
  private float rotationX, rotationY, rotationZ;
  private Vector3 shipPos   = Vector3.Zero,
                  cameraPos = new Vector3( 0f, 500f,  3000f ),
                  camLookAt = new Vector3( 0f, 500f, -3000f );

  public Game1()
  { g = new GraphicsDeviceManager( this );
    content  = new ContentManager( Services );
  }
  protected override void Initialize()
  { g.IsFullScreen = false;
    Window.Title = "SkyBox";
    Window.AllowUserResizing = true;
    base.Initialize();
  }
  protected override void LoadContent()
  { skyModel   = content.Load< Model >( "Content\\skybox"  );
    shipModel  = content.Load< Model >( "Content\\ship"  );
    //skyModel just has 1 effect whereas shipModel has 3 effects.
    skyEffect  = (BasicEffect)skyModel .Meshes[0].Effects[0];
  }
  protected override void UnloadContent()
  { content.Unload();
  }
  protected override void Update( GameTime gameTime )
  { GamePadState s = GamePad.GetState( PlayerIndex.One );
    if ( !s.IsConnected ) Exit();
    ButtonState bp = ButtonState.Pressed;
    if ( s.Buttons.Back  == bp )
    { shipPos = Vector3.Zero;
      cameraPos = new Vector3( 0f, 500f,  3000f );
      camLookAt = new Vector3( 0f, 500f, -3000f );
      rotationX = rotationY = rotationZ = 0f;
    }
    if ( s.DPad.Left ==bp ) { cameraPos.X-=slow; camLookAt.X-=slow; shipPos.X-=slow; }
    if ( s.DPad.Right==bp ) { cameraPos.X+=slow; camLookAt.X+=slow; shipPos.X+=slow; }
    if ( s.DPad.Up   ==bp ) { cameraPos.Y+=slow; camLookAt.Y+=slow; shipPos.Y+=slow; }
    if ( s.DPad.Down ==bp ) { cameraPos.Y-=slow; camLookAt.Y-=slow; shipPos.Y-=slow; }

    Vector2 ts = s.ThumbSticks.Left;  rotationZ-=0.1f*ts.X; rotationX-=0.1f*ts.Y;
            ts = s.ThumbSticks.Right; rotationZ-=0.1f*ts.X; rotationY-=0.1f*ts.Y;
    shipworldM = Matrix.CreateTranslation( shipPos );
    skyworldM  = Matrix.CreateScale( skyscale , skyscale , skyscale  );
    viewM      = Matrix.CreateTranslation( -shipPos.X, -shipPos.Y, -shipPos.Z ) *
                 Matrix.CreateRotationX( rotationX ) *
                 Matrix.CreateRotationY( rotationY ) *
                 Matrix.CreateRotationZ( rotationZ ) *
                 Matrix.CreateLookAt( cameraPos, camLookAt, Vector3.Up ) * shipworldM;
    projM = Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi/3, 1f, 1f, 10f*skyscale );
    g.GraphicsDevice.RenderState.CullMode = CullMode.None;
    //g.GraphicsDevice.RenderState.DepthBufferEnable = false;
    //g.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
    base.Update( gameTime );
  }
  protected override void Draw( GameTime gameTime )
  { g.GraphicsDevice.Clear( Color.DarkBlue );
    skyEffect.World      = skyworldM;
    skyEffect.View       = viewM;
    skyEffect.Projection = projM;
    skyModel .Meshes[0].Draw();
    foreach ( BasicEffect effect in shipModel.Meshes[0].Effects )
    { effect.EnableDefaultLighting();
      effect.World      = shipworldM;
      effect.View       = viewM;
      effect.Projection = projM;
      shipModel.Meshes[0].Draw();
    }
  }
} // end of class Game1
} // end of class Program